# -*- coding: utf-8 -*-

"""
fabrik.paths
------------------
This module generates the various paths used by fabrik.
"""


from fabric.state import env
from unipath import Path


def get_deploy_path(child=None):
    path = Path(env.app_path)

    if child:
        path = Path(path, child)

    return path


def get_current_release_path(child=None):
    release_name = get_current_release_name()

    if not release_name:
        return

    path = get_releases_path(release_name)

    if child:
        path = Path(path, child)

    return path


def get_current_release_name():
    run_args = {}

    # Append capture value if we are running locally
    if env.run.__name__ == "elocal":
        run_args["capture"] = True

    path = env.run(
        "ls -dt %s/*/ | sort -n -t _ -k 2 | tail -1" %
        get_releases_path(), **run_args)

    if not path:
        return

    release = Path(path)

    try:
        int(release.absolute().name)
    except ValueError, e:
        print e
        return

    return release.absolute().name


def get_releases_path(child=None):
    return _path_optional(get_deploy_path(), "releases", child)


def get_shared_path(child=None):
    return _path_optional(get_deploy_path(), "shared", child)


def get_backup_path(child=None):
    return _path_optional(get_deploy_path(), "backup", child)


def get_current_path(child=None):
    return _path_optional(get_deploy_path(), "current", child)


def get_upload_path(child=None):
    return _path_optional(get_deploy_path(), "upload", child)


def get_source_path(release):
    path = get_releases_path(release)

    if "source_path" in env:
        path = Path(path, env.source_path)

    return path


def _path_optional(base, sub, child=None):
    path = Path(base, sub)

    if child:
        path = Path(path, child)

    return path


def symlink(origin, dest):
    env.run("ln -nsf %s %s " % (origin, dest))



# -*- coding: utf-8 -*-

"""
fabrik.ext.composer
-------------------------
Contains methods for syncing composer packages
"""

from fabric.decorators import task
from fabric.state import env
from fabrik import paths


@task
def install(install_path=None):
    if not install_path:
        install_path = paths.get_current_release_path()

    with(env.cd(install_path)):
        env.run("curl -sS https://getcomposer.org/installer | php")


@task
def update(install_path=None):
    if not install_path:
        install_path = paths.get_current_release_path()

    with(env.cd(install_path)):
        env.run("php composer.phar install --no-scripts --optimize-autoloader")



# -*- coding: utf-8 -*-

"""
fabrik.recipes.django_uwsgi
---------------------------------
This recipe is based on django, but introduces a uwsgi reload on after_deploy
"""

from fabrik.hooks import hook
from fabrik.ext import uwsgi
from fabrik.recipes import django


@hook("setup")
def setup():
    # TODO: Copy .ini file from templates to server
    pass


@hook("after_deploy")
def after_deploy():
    uwsgi.service_restart()




# -*- coding: utf-8 -*-

"""
fabrik.transfer.git
-------------------------
Sets up a standard git clone procedure.

Params:
    git_passphrase: Remote server git ssh passphrase (Optional)
"""

from fabric.state import env
from fabric.context_managers import settings
from fabrik.hooks import hook
from ..logger import logger


@hook("copy")
def copy():
    branch = None

    if "branch" in env:
        branch = env.branch
    else:
        logger.warn("Git branch not set, using master instead")
        branch = "master"

    with apply_settings(), env.cd(env.app_path):
        env.run("git clone  --depth 1 -b %(branch)s %(repro)s %(path)s" % {
            "branch": branch,
            "repro": env.repro_url,
            "path": env.current_release
        })


def apply_settings():
    """
    Applies additional settings before clone takes place"
    """

    prompts = {}

    if "git_passphrase" in env:
        prompts["': "] = env.git_passphrase

    return settings(prompts=prompts)



# -*- coding: utf-8 -*-


"""
fabrik.recipes.vendors.glesys_wordpress
---------------------------------------------
Wordpress recipe for Glesys Wordpress managed hosting.
"""

from fabrik.recipes import wordpress
from fabrik.ext import mysql


__version__ = "1.0.0"


@hook("before_deploy")
def backup_db():
    mysql.backup_db()



# -*- coding: utf-8 -*-

"""
fabrik.ext.virtualenv
---------------------------
Holds methods for managing virtualenv.
"""

import os
from unipath import Path
from fabric.decorators import task
from fabric.operations import prompt
from fabric.state import env

from fabrik import paths
from ..logger import logger


@task
def create_venv():
    if "venv_path" not in env:
        raise Exception("No env.venv_path has been specified")

    # Checks if venv exist and prompts user if reinstalling is an option
    if env.exists(Path(get_path(), "bin")):
        logger.warn("Virtualenv is already installed")
        prompt_result = prompt("Install anyways:", default="no")

        if prompt_result == "no":
            return

    env.run("virtualenv %s" % get_path())


@task
def update_requirements():
    req_dir = None
    req_path = None

    if "requirements_root" in env:
        req_dir = os.path.join(env.current_release, env.requirements_root)

    if not req_dir:
        req_dir = os.path.join(env.current_release, "requirements")

    if "requirements" in env:
        req_path = Path(req_dir, env.requirements)

    if not req_path:
        req_path = Path(req_dir, "%s.txt" % env.stage)

    if not env.exists(req_path):
        raise Exception("Requirement file not found at %s" % req_path)

    env.run("pip install -r %s" % req_path)


def get_path():
    if "venv_path" in env:
        return env.venv_path

    return paths.get_deploy_path("venv")



# -*- coding: utf-8 -*-

"""
fabrik.ext.envfile
------------------
Contains .env-file helpers
"""

from fabric.decorators import task
from fabric.state import env
from fabrik import paths


@task
def create_env():
    env.run("touch %s" % paths.get_shared_path(".env"))
    env.run("chmod 400 %s" % paths.get_shared_path(".env"))


@task
def symlink_env():
    paths.symlink(
        paths.get_shared_path(".env"),
        paths.get_current_path(".env")
    )



# -*- coding: utf-8 -*-

"""
fabrik.ext.postgres
-------------------
A PostgreSQL extension that handles backups and remote/local syncing

Params:
    psql_user
    psql_db
    psql_password

    local_psql_user
    local_psql_db
    local_psql_password

Commands:
    backup_db
    restore_db
    sync_local_to_remote
    sync_remote_to_local
"""

import time
from fabric.decorators import task
from fabric.state import env
from fabric.api import get, put
from fabric import context_managers
from fabric.operations import prompt

from fabrik import paths
from fabrik.logger import logger
from fabrik.api import init_tasks
from fabrik.utils.elocal import elocal
from fabrik.hooks import run_hook, has_hook


def _check_requirements():
    assert "psql_user" in env, "Missing psql_user in env"
    assert "psql_db" in env, "Missing psql_db in env"
    assert "psql_password" in env, "Missing psql_password in env"

    assert "local_psql_user" in env, "Missing local_psql_user in env"
    assert "local_psql_db" in env, "Missing local_psql_db in env"
    assert "local_psql_password" in env, "Missing local_psql_password in env"


@task
def backup_db(release=None, limit=5):
    """
    Backup database and associate it with current release
    """

    assert "psql_user" in env, "Missing psql_user in env"
    assert "psql_db" in env, "Missing psql_db in env"
    assert "psql_password" in env, "Missing psql_password in env"

    if not release:
        release = paths.get_current_release_name()

    max_versions = limit+1

    if not release:
        logger.info("No releases present, skipping task")
        return

    remote_file = "postgresql/%s.sql.tar.gz" % release
    remote_path = paths.get_backup_path(remote_file)

    env.run("mkdir -p %s" % paths.get_backup_path("postgresql"))

    with context_managers.shell_env(PGPASSWORD=env.psql_password):
        env.run("pg_dump -h localhost -Fc -f %s -U %s %s -x -O" % (
            remote_path, env.psql_user, env.psql_db
        ))

    # Remove older releases
    env.run("ls -dt %s/* | tail -n +%s | xargs rm -rf" % (
        paths.get_backup_path("postgresql"),
        max_versions)
    )


@task
def restore_db(release=None):
    """
    Restores backup back to version, uses current version by default.
    """

    if not release:
        release = paths.get_current_release_name()

    if not release:
        raise Exception("Release %s was not found" % release)

    backup_file = "postgresql/%s.sql.gz" % release
    backup_path = paths.get_backup_path(backup_file)

    if not env.exists(backup_path):
        raise Exception("Backup file %s not found" % backup_path)

    with context_managers.shell_env(PGPASSWORD=env.psql_password):
        env.run("pg_restore --clean -h localhost -d %s -U %s '%s'" % (
            env.psql_db,
            env.psql_user,
            backup_path)
        )


@task
def sync_local_to_remote(force="no"):
    """
    Sync your local postgres database with remote

    Example:
        fabrik prod sync_local_to_remote:force=yes
    """

    _check_requirements()

    if force != "yes":
        message = "This will replace the remote database '%s' with your "\
            "local '%s', are you sure [y/n]" % (env.psql_db, env.local_psql_db)
        answer = prompt(message, "y")

        if answer != "y":
            logger.info("Sync stopped")
            return

    init_tasks()  # Bootstrap fabrik

    # Create database dump
    local_file = "sync_%s.sql.tar.gz" % int(time.time()*1000)
    local_path = "/tmp/%s" % local_file

    with context_managers.shell_env(PGPASSWORD=env.local_psql_password):
        elocal("pg_dump -h localhost -Fc -f %s -U %s %s -x -O" % (
            local_path, env.local_psql_user, env.local_psql_db
        ))

    remote_path = "/tmp/%s" % local_file

    # Upload sync file
    put(remote_path, local_path)

    # Import sync file by performing the following task (drop, create, import)
    with context_managers.shell_env(PGPASSWORD=env.psql_password):
        env.run("pg_restore --clean -h localhost -d %s -U %s '%s'" % (
            env.psql_db,
            env.psql_user,
            remote_path)
        )

    # Cleanup
    env.run("rm %s" % remote_path)
    elocal("rm %s" % local_path)

    # Trigger hook
    run_hook("postgres.after_sync_local_to_remote")

    logger.info("Sync complete")


@task
def sync_remote_to_local(force="no"):
    """
    Sync your remote postgres database with local

    Example:
        fabrik prod sync_remote_to_local
    """

    _check_requirements()

    if force != "yes":
        message = "This will replace your local database '%s' with the "\
            "remote '%s', are you sure [y/n]" % (env.local_psql_db, env.psql_db)
        answer = prompt(message, "y")

        if answer != "y":
            logger.info("Sync stopped")
            return

    init_tasks()  # Bootstrap fabrik

    # Create database dump
    remote_file = "postgresql/sync_%s.sql.tar.gz" % int(time.time()*1000)
    remote_path = paths.get_backup_path(remote_file)

    env.run("mkdir -p %s" % paths.get_backup_path("postgresql"))

    with context_managers.shell_env(PGPASSWORD=env.psql_password):
        env.run("pg_dump -h localhost -Fc -f %s -U %s %s -x -O" % (
            remote_path, env.psql_user, env.psql_db
        ))

    local_path = "/tmp/%s" % remote_file

    # Download sync file
    get(remote_path, local_path)

    # Import sync file by performing the following task (drop, create, import)
    with context_managers.shell_env(PGPASSWORD=env.local_psql_password):
        elocal("pg_restore --clean -h localhost -d %s -U %s '%s'" % (
            env.local_psql_db,
            env.local_psql_user,
            local_path)
        )

    # Cleanup
    env.run("rm %s" % remote_path)
    elocal("rm %s" % local_path)

    # Trigger hook
    run_hook("postgres.after_sync_remote_to_local")

    logger.info("Sync complete")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import os
import fabric.main


def main():
    # Supply default fabricrc (if one does not exist)
    if "-c" not in sys.argv:
        rc_files = ("fabricrc.txt", ".fabricrc")

        for path in rc_files:
            if os.path.exists(path):
                sys.argv.extend(("-c", path))
                break

    fabric.main.main()



# -*- coding: utf-8 -*-

"""
fabrik.utils
------------------
Contains various utility function, such as hooks and way of retriving config
"""

from fabric.task_utils import crawl
from fabric.api import execute
from fabric import state
from fabric.state import env
from ..logger import logger


def run_task(task):
    """
    A method of running fabric task with silent errors.
    """

    if has_task(task):
        execute(task)


def has_task(task):
    """
    Checks if fabric task exists
    """

    return crawl(task, state.commands) is not None


def get_stage_var(name, default=None):
    if "stage" not in env:
        raise Exception("env.stage cannot be empty")

    key = "%s_%s" % (env.stage.upper(), name)

    if default is None:
        return env["%s_%s" % (env.stage.upper(), name)]

    return env.get(key, default)


def get_var(name):
    return env[name]



# -*- coding: utf-8 -*-

"""
fabrik.hooks
------------------
A simple hook subscription utility, used when triggering recipe tasks.
"""

from functools import wraps
import inspect


_hooks = {}


def hook(name=None, priority=-1):
    """
    Decorator
    """

    def _hook(hook_func):
        return register_hook(name, hook_func=hook_func, priority=priority)

    return _hook


def register_hook(name, hook_func=None, priority=-1):
    if name not in _hooks:
        _hooks[name] = []

    hook_list = _hooks[name]

    if priority == -1:
        priority = len(hook_list)

    def _decorator(*args, **kwargs):
        return hook_func(*args, **kwargs)

    wrap = wraps(hook_func)(_decorator)
    wrap.hook_id = _get_hook_id(hook_func)

    hook_list.insert(min(len(hook_list), priority), wrap)

    return wrap


def _get_hook_id(hook_func):
    package_name = inspect.getmodule(hook_func).__name__
    full_path = "%s.%s" % (package_name, hook_func.__name__)

    return full_path


def run_hook(name, *args, **kwargs):
    if not has_hook(name):
        return

    hook_list = _hooks[name]
    for function in hook_list:
        function(*args, **kwargs)


def has_hook(name, hook_func=None):
    if name not in _hooks:
        return False

    if len(_hooks[name]) == 0:
        return False

    if not hook_func:
        return True

    for func in _hooks[name]:
        hook_id = _get_hook_id(hook_func)

        if hook_id == func.hook_id:
            return True

    return False


def unregister_hook(name, hook_func=None):
    if not has_hook(name, hook_func=hook_func):
        return

    hook_id = _get_hook_id(hook_func)

    for func in _hooks[name][:]:
        if hook_id == func.hook_id:
            _hooks[name].remove(func)



# -*- coding: utf-8 -*-

"""
fabrik.utils.erun
-----------------------
Fabric "run" command that raises an exception
"""

from fabric.api import run, settings


class RunFailedCommand(Exception):
    def __init__(self, message, result):
        Exception.__init__(self, message)
        self.result = result


def erun(*args, **kwargs):
    with settings(warn_only=True):
        result = run(*args, **kwargs)
        if result.failed:
            raise RunFailedCommand("args: %r, kwargs: %r, error code: %r"
                                   % (args, kwargs, result.return_code), result)
        return result




# -*- coding: utf-8 -*-

"""
fabrik.ext.uwsgi
----------------------
Methods for handling the uwsgi server.
"""

from fabric.state import env


def restart():
    env.run("service uwsgi restart")


# This is deprecated, use touch_restart() instead
def touch_reload():
    touch_restart()


def touch_restart():
    env.run("touch %s" % env.uwsgi_ini_path)


def service_reload():
    env.run("service uwsgi reload")


def service_restart():
    env.run("service uwsgi restart")



# -*- coding: utf-8 -*-

"""
fabrik.ext.wpcli
------------------
A wp-cli extension that contains the following tasks:

Params:
    local_wp_dir

Commands:
    sync_remote_to_local
"""

import time
from fabric.decorators import task
from fabric.api import get
from fabric.context_managers import lcd
from fabric.state import env
from fabric.operations import prompt

from fabrik import paths
from fabrik.logger import logger
from fabrik.api import init_tasks
from fabrik.utils.elocal import elocal


@task
def sync_remote_to_local(force="no"):
    """
    Replace your remote db with your local

    Example:
        sync_remote_to_local:force=yes
    """

    assert "local_wp_dir" in env, "Missing local_wp_dir in env"

    if force != "yes":
        message = "This will replace your local database with your "\
            "remote, are you sure [y/n]"
        answer = prompt(message, "y")

        if answer != "y":
            logger.info("Sync stopped")
            return

    init_tasks()  # Bootstrap fabrik

    remote_file = "sync_%s.sql" % int(time.time()*1000)
    remote_path = "/tmp/%s" % remote_file

    with env.cd(paths.get_current_path()):
        env.run("wp db export %s" % remote_path)

    local_wp_dir = env.local_wp_dir
    local_path = "/tmp/%s" % remote_file

    # Download sync file
    get(remote_path, local_path)

    with lcd(local_wp_dir):
        elocal("wp db import %s" % local_path)

    # Cleanup
    env.run("rm %s" % remote_path)
    elocal("rm %s" % local_path)



# -*- coding: utf-8 -*-

"""
fabrik.ext.forever
-------------------------
Contains methods for dealing with nodejs forever

Api doc: https://github.com/nodejitsu/forever
"""

from fabric.decorators import task
from fabric.state import env
from fabrik import paths


@task
def restart():
    if "forever_app" not in env:
        raise Exception("Could not find app to run"
                "env.forever_app must be set")

    env.run("forever start %s" % env.forever_app)


def restart_service():
    env.run("service forever restart")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import pip

from setuptools import setup, find_packages
import fabrik


if sys.argv[-1] == "publish":
    os.system("python setup.py sdist upload")
    sys.exit()

package_exclude = ("tests*", "examples*")
packages = find_packages(exclude=package_exclude)

# Handle requirements
install_requires = [
]

tests_require = install_requires


# Convert markdown to rst
try:
    from pypandoc import convert
    long_description = convert("README.md", "rst")
except:
    long_description = ""


setup(
    name="fabrik",
    version=fabrik.__version__,
    description=("A simple to use deployment toolkit built on top of Fabric"),
    long_description=long_description,
    author="Fröjd",
    author_email="martin.sandstrom@frojd.se",
    url="https://github.com/frojd/fabrik",
    packages=packages,
    include_package_data=True,
    install_requires=[
        'Fabric==1.10.0',
        'Unipath==1.0',
    ],
    tests_require=[
        'Fabric==1.10.0',
        'Unipath==1.0',
    ],
    entry_points={
        "console_scripts": [
            "fabrik = fabrik.scripts.fabrik:main",
        ]
    },
    license="MIT",
    zip_safe=False,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Topic :: Software Development :: Build Tools",
        "Topic :: Software Development :: Libraries",
        "Topic :: System :: Software Distribution",
        "Topic :: System :: Systems Administration",
    ],
)



# -*- coding: utf-8 -*-

"""
fabrik.utils.elocal
-------------------------
"local" command that raises an exception
Credit to https://gist.github.com/lost-theory/1831706
"""

from fabric.api import local, settings


class LocalFailedCommand(Exception):
    def __init__(self, message, result):
        Exception.__init__(self, message)
        self.result = result


def elocal(*args, **kwargs):
    with settings(warn_only=True):
        result = local(*args, **kwargs)
        if result.failed:
            raise LocalFailedCommand("args: %r, kwargs: %r, error code: %r"
                                     % (args, kwargs, result.return_code),
                                     result)
        return result



# -*- coding: utf-8 -*-

"""
fabrik.ext.celeryd
------------------
A collection of celeryd helpers.
"""

from fabric.decorators import task
from fabric.state import env


@task
def restart_celery():
    env.run("service celeryd restart")
